// FILE: src/services/ZhFilterBuilder.js
/**
 * Service to create the filter conditions
 *
 * @class
 * @since 3.0.0
 *
 * @param {QueryBuilder} Query Service to build the query conditions
 */
function ZhFilterBuilder(Query){

    var defaultOperators = {
        'field/between-dates.html': Query.BETWEEN,
        'field/calendar.html': Query.EQ,
        'field/date.html': Query.EQ,
        'field/datetime.html': Query.EQ,
        'field/interval.html': {
            'default': Query.BETWEEN,
            'start': Query.GTE,
            'end': Query.LTE
        },
        'field/number-edit.html': Query.EQ,
        'field/number-keyboard.html': Query.EQ,
        'field/number.html': Query.EQ,
        'field/radio.html': Query.EQ,
        'field/select.html': Query.EQ,
        'field/select-simple.html': Query.EQ,
        'field/select-native.html': Query.EQ,
        'field/select-autocomplete.html': Query.EQ,
        'field/select-multiple.html': Query.IN,
        'default': Query.LIKE_INSENSITIVE
    };

    var buildFilterByTemplate = {
        'field/select.html': buildSelectFilter,
        'field/select-autocomplete.html': buildSelectFilter,
        'field/select-native.html': buildSelectNativeFilter,
        'field/select-multiple.html': buildSelectMultipleFilter,
        'field/interval.html':buildIntervalFilter,
        'default': buildDefaultFilter
    };

    /**
     * Register the default operator for a field
     *
     * @since 3.0.0
     *
     * @param {String} template Template of the field
     * @param {String} operator Operator to use on template
     */
    this.setDefaultOperator = function(template, operator){
        defaultOperators[template] = operator;
    };

    /**
     * Register the method used to create condition for a field
     *
     * @since 3.0.0
     *
     * @param {String}   template Template of the field
     * @param {Function} callback Method to be used to create the condition
     */
    this.setBuildFilterByTemplate = function(template, callback){
        buildFilterByTemplate[template] = callback;
    };

    function getFieldTemplate(field){
        return field.template.split('#').pop();
    }

    function getFilterBuilderByTemplate(template){
        if(!buildFilterByTemplate[template]){
            template = 'default';
        }

        return buildFilterByTemplate[template];
    }

    /**
     * Clear filter with same filter name
     *
     * @since 3.10.0
     *
     * @param  {Object[]} List of filters
     *
     * @return {Object[]} List of filters with unique name
     */
    function clearFilters(filters){
        var clearedFilters = [];
        filters.forEach(function(filter){
            var repeated = clearedFilters.some(function(_filter){
                return _filter.name === filter.name;
            });

            if(!repeated){
                clearedFilters.push(filter);
            }
        });
        return clearedFilters;
    }

    /**
     * Build filter conditions
     *
     * @since 3.0.0
     *
     * @param  {Field[]} fields List of fields to get the filter values
     * @param  {Object}  row    Row to get the values from
     *
     * @return {Object[]}       List of conditions
     */
    this.buildFilter = function (fields, row){
        var query = Query.build();

        fields.forEach(function(field){
            if(!field.skipSearch){
                var template = getFieldTemplate(field);
                getFilterBuilderByTemplate(template)(query, field, row);
            }
        });

        return clearFilters(query.where());
    };

    var operatorFlow = {
        "BETWEEN": function (query, value){
            query.between([value.start, value.end]);
        },
        "NOT BETWEEN": function (query, value){
            query.notBetween([value.start, value.end]);
        },
        "LIKE": function (query, value){
            query.like('%' + value + '%');
        },
        "LIKE_I": function (query, value){
            query.likeInsensitive('%' + value + '%');
        },
        "LIKE_BEGINNING": function (query, value){
            query.likeInsensitive( value + '%');
        },
        "NOT LIKE": function (query, value){
            query.notLike('%' + value + '%');
        },
        "LIKE ALL": function (query, value){
            query.likeAll('%' + value + '%');
        },
        "<>": function (query, value){
            query.notEquals(value);
        },
        "=": function (query, value){
            query.equals(value);
        },
        ">": function (query, value){
            query.greater(value);
        },
        "<": function (query, value){
            query.lower(value);
        },
        ">=": function (query, value){
            query.greaterOrEqual(value);
        },
        "<=": function (query, value){
            query.lowerOrEqual(value);
        },
        "IN": function (query, value){
            query.in(value);
        },
        "NOT IN": function (query, value){
            query.notIn(value);
        }
    };

    function buildFilter(query, fieldName, operator, value){
        query.where(fieldName);
        operatorFlow[operator](query, value);
    }

    function isEmptyArrayOrObject(obj){
        var name;
        for(name in obj){
            return false;
        }
        return true;
    }

    function existValue(value){
        var isObject = typeof value == 'object';
        return isObject ? !isEmptyArrayOrObject(value) : !Util.isEmptyOrBlank(value);
    }

    function buildDefaultFilter(query, field, currentRow){
        var fieldName = field.name;
        var value = currentRow[fieldName];

        if(existValue(value)){
            var operator = getFieldOperator(field);
            buildFilter(query, fieldName, operator, value);
        }
    }

    function buildIntervalFilter(query, field, currentRow){
        var fieldName = field.name;
        var value = currentRow[fieldName];
        value = existValue(value) ? Util.clone(value) : {};

        var existStartValue = !Util.isEmptyOrBlank(value.start);
        var existEndValue = !Util.isEmptyOrBlank(value.end);

        if(existStartValue || existEndValue){
            var operator = getFieldOperator(field);
            if(existStartValue && existEndValue){
                operator = operator.default;
            } else if(existStartValue){
                operator = operator.start;
                value = value.start;
            } else {
                operator = operator.end;
                value = value.end;
            }
            buildFilter(query, fieldName, operator, value);
        }
    }

    function buildSelectWithValueFieldFilter(query, field, currentRow, operator){
        var fieldName = field.valueField;
        var value = currentRow[fieldName];
        if(existValue(value)){
            buildFilter(query, fieldName, operator, value);
        }
    }

    function buildSelectWithValueFieldsFilter(query, field, currentRow, operator){
        var fieldName = field.name;
        var value = currentRow[fieldName];

        if(existValue(value)){
            buildFilter(query, fieldName, operator, value);
        }
    }

    function buildSelectWithOutDataFilter(query, field, currentRow, operator){
        var outData = field.outData;
        Object.keys(outData).forEach(function(outDataKey){
            var fieldName = outData[outDataKey];
            var value = currentRow[fieldName];
            if(existValue(value)){
                buildFilter(query, fieldName, operator, value);
            }
        });
    }

    function buildSelectFilter(query, field, currentRow){
        var operator = getFieldOperator(field);
        if(field.valueField){
            buildSelectWithValueFieldFilter(query, field, currentRow, operator);
        } else if(field.valueFields){
            buildSelectWithValueFieldsFilter(query, field, currentRow, operator);
        } else if(field.outData){
            buildSelectWithOutDataFilter(query, field, currentRow, operator);
        } else {
            throw new Error('Invalid select configuration on field ' + field.name);
        }
    }

    function buildSelectNativeWithValueFieldFilter(query, field, currentRow, operator){
        var fieldName = field.name;
        var value = currentRow[fieldName];
        if(existValue(value)){
            buildFilter(query, fieldName, operator, value);
        }
    }

    function buildSelectNativeFilter(query, field, currentRow){
        var operator = getFieldOperator(field);

        if(field.valueField){
            buildSelectNativeWithValueFieldFilter(query, field, currentRow, operator);
        } else {
            throw new Error('Invalid select-native configuration on field ' + field.name);
        }

    }

    function buildSelectMultipleWithValueFieldFilter(query, field, currentRow, operator){
        var fieldName = field.name;
        var value = currentRow[fieldName];
        if(currentRow[fieldName + '_EXCEPT']){
            value = currentRow[fieldName + '_EXCEPT'];
            operator = 'NOT IN';
        }

        if(existValue(value)){
            buildFilter(query, fieldName, operator, value);
        }
    }

    function buildSelectMultipleWithValueFieldsFilter(query, field, currentRow, operator){
        throw new Error('Filter on field select-multiple with property valuesFields is not supported yet. ' +
            'Field "'+field.name+'"');
    }

    function buildSelectMultipleFilter(query, field, currentRow){
        var operator = getFieldOperator(field);

        if(field.valueField){
            buildSelectMultipleWithValueFieldFilter(query, field, currentRow, operator);
        } else if(field.valueFields){
            buildSelectMultipleWithValueFieldsFilter(query, field, currentRow, operator);
        } else {
            throw new Error('Invalid select-multiple configuration on field ' + field.name);
        }
    }

    function getFieldOperator(field){
        var template = getFieldTemplate(field);
        return field.operator || defaultOperators[template] || defaultOperators['default'];
    }

}

Configuration(function(ContextRegister){
    ContextRegister.register('ZhFilterBuilder', ZhFilterBuilder);
});

// FILE: src/services/ZhFilterDescriptionBuilder.js
/**
 * Service to create the applied filter description
 *
 * @class
 * @since 3.0.0
 *
 * @param {DecodeValueFieldService} DecodeValueFieldService Service to retrieve field value
 */
function ZhFilterDescriptionBuilder(DecodeValueFieldService, ZhUtil, templateManager){

    /**
     * Build filter description for a filter preference
     *
     * @since 3.0.0
     *
     * @param  {String} preferenceName Filter preference label
     *
     * @return {Object[]}              List containing the applied preference description
     */
    this.buildFilterAppliedInfo = function(preferenceName){
        return [{
            'label': 'Filter',
            'filterName': preferenceName
        }];
    };

    function getFilterWidget(id){
        return templateManager.container.factoredWidgets.filter(function(widget){ return widget._filteredWidgetId == id; })[0];
    }

    /**
     * Build filter description for a list of filter conditions
     *
     * @since 3.0.0
     *
     * @param  {Field[]} fields         List of fields on the list widget
     * @param  {Object}  currentRow     Row of the applied filter
     * @param  {Object}  searchCriteria Search criteria currently applied
     *
     * @return {Object[]}               List of descriptions for the applied conditions
     */
    this.buildFilterInfo = function(fields, currentRow, searchCriteria, widget){
        var filterInfo = [];

        var filterWidget = getFilterWidget(widget && widget.id);

        fields.forEach(function(field){
            buildDefaultFilterInfo(filterInfo, field, currentRow, filterWidget);
        });

        buildFilterInfoForSearchCriteria(fields, filterInfo, searchCriteria);

        return filterInfo;
    };

    function buildFilterInfoForSearchCriteria(fields, filterInfo, searchCriteria){
        if(searchCriteria){
            Object.keys(searchCriteria).forEach(function(fieldName){
                if(searchCriteria[fieldName] !== ''){
                    var fieldLabel;
                    if(fieldName == '_ALL_MAP'){
                        fieldLabel = 'All fields';
                    } else {
                        fieldLabel = getFieldByName(fields, fieldName).label;
                    }

                    filterInfo.push(buildFilterInfo(fieldLabel, searchCriteria[fieldName]));
                }
            });
        }
    }

    function getFieldByName(fields, fieldName){
        return fields.filter(function(field){
            return field.name === fieldName;
        }).shift();
    }

    function buildFilterInfo(label, value){
        return {
            'label': label,
            'value': value
        };
    }

    function buildDefaultFilterInfo(filterInfo, field, currentRow, widget){
        var condition = !Util.isEmptyOrBlank(currentRow[field.name]);
        var filterField = widget && widget.getField(field.name);
        var showInfo = filterField && filterField.isVisible;

        if(condition && showInfo){
            var value = DecodeValueFieldService.model(field, currentRow[field.name]);
            value = ZhUtil.evaluate(value);
            filterInfo.push(buildFilterInfo(field.label, value));
        }
    }

}

Configuration(function(ContextRegister){
    ContextRegister.register('ZhFilterDescriptionBuilder', ZhFilterDescriptionBuilder);
});

// FILE: src/services/ZhFilterPreferences.js
/**
 * A filter preference
 * @typedef {Object} Filter
 * @property {String} ID                  Filter unique identifier
 * @property {String} USER_ID             User identifier that created the filter
 * @property {String} ORGANIZATION_ID     Organization the user belongs to
 * @property {String} LABEL_TO_PREFERENCE Filter label
 * @property {String} WIDGET_ID           Widget identifier the filter belongs to
 * @property {String} FILTER_ROW          Row indicating the filter settings
 * @property {String} LAST_MODIFIED_DATE  Last date the filter was modified
 */

/**
 * Service to handle filters storage
 *
 * @class
 * @since 3.0.0
 *
 * @param {ZhPreferences}              ZhPreferences              Service to handle preferences storage
 * @param {ZhFilterBuilder}            ZhFilterBuilder            Service to create filters conditions
 * @param {ZhFilterDescriptionBuilder} ZhFilterDescriptionBuilder Service to conditions description
 * @param {TemplateManager}            templateManager            Service to handle template
 * @param {EventAggregator}            eventAggregator            Service to use project events
 * @param {ZhUtil}                     ZhUtil                     Service to provice utilities methods
 * @param {EventEngine}                eventEngine                Service to use models events
 * @param {MetaDataFactory}            metaDataFactory            Service to factory models from metadata
 * @param {ZHPromise}                  ZHPromise                  Service to handle promises
 * @param {MessageService}             MessageService             Service to handle messages
 * @param {ScreenService}              ScreenService              Service to handle screen
 * @param {ZhKeyboardService}          ZhKeyboardService          Service to handle keyboard binds
 * @param {ApplicationContext}         ApplicationContext         Service to retrieve Zeedhi services
 */
function ZhFilterPreferences(ZhPreferences, ZhFilterBuilder, ZhFilterDescriptionBuilder, templateManager, eventAggregator, ZhUtil, eventEngine, metaDataFactory, ZHPromise, MessageService, ScreenService, ZhKeyboardService, ApplicationContext, $timeout) {
    var self = this;
    var originalFilterRows = {};

    var originalFilter = {
        'ID': 'ORIGINAL',
        'USER_ID': 0,
        'ORGANIZATION_ID': 0,
        'LABEL_TO_PREFERENCE': 'Original'
    };

    function shouldShowFilterAction(widget){
        return Util.get(widget, ['floatingControl', 'searchAction']) !== false ||
            Util.get(widget, ['floatingControl', 'customizationAction', 'filter']) !== false;
    }

    eventAggregator.subscribe('beforeProjectStartup', function(){
        ZhPreferences.registerRepository('zh-filter-preferences#filter');

        templateManager.project.customFloatingActions.push({
            'name': 'Filter Action',
            'label': 'Search',
            'type': 'filter',
            'order': 20,
            'icon': 'search',
            'template': 'zh-filter-preferences#floating-filter-card.html',
            'expandable': true,
            'isVisible': shouldShowFilterAction
        });
    });

    eventEngine.registerEvent('FilterBeforeApply', 'beforeFilterApply');
    eventEngine.registerEvent('FilterAfterApply', 'afterFilterApply');

    var filterRegisteredDataSources = {};

    /**
     * Register filters datasource
     *
     * @since 3.0.0
     *
     * @param {String} widgetID         Widget identifier
     * @param {Object} datasource       Datasource that contains the filters
     *
     */
    this.registerFilterDataSource = function(widgetID, dataSource){
        if(widgetID){
            filterRegisteredDataSources[widgetID] = dataSource;
        }

        return filterRegisteredDataSources;
    };

    /**
     * Update dataSource with local saved filters
     *
     * @since 3.0.0
     *
     * @param  {String}     widgetID   Widget identifier
     * @param  {DataSource} dataSource DataSource that contains the loaded filters
     *
     * @return {Promise}               Promise resolved after the filters are loaded into the dataSource
     */
    this.updateFilters = function(widgetID, dataSource){
        if(!dataSource){
            dataSource = filterRegisteredDataSources[widgetID];
        }

        return this.getWidgetFilters(widgetID).then(function(filters){
            return this.addIsDefaultFilterToFilters(filters, widgetID).then(function(filters){
                dataSource.data = filters;
            });
        }.bind(this));
    };

    function addIsDefaultFilterToFilter(defaultFilter, filter){
        filter.DEFAULT_FILTER = filter.ID === defaultFilter;
        return filter;
    }

    /**
     * Add property to identify if the filter is selected as default
     *
     * @since 3.0.0
     *
     * @param {Filter[]} filters  List of filters to update the default filter property
     * @param {String}   widgetID Widget to get the default properties
     */
    this.addIsDefaultFilterToFilters = function(filters, widgetID){
        return ZhPreferences.getDefaultWidgetPreferences(widgetID).then(function(defaultPreferences){
            var defaultFilter = defaultPreferences.DEFAULT_FILTER;
            return filters.map(addIsDefaultFilterToFilter.bind(this, defaultFilter));
        });
    };

    /**
     * Retrieve local cached filters for specified widget
     *
     * @since 3.0.0
     *
     * @param  {String} widgetID   Widget identifier
     *
     * @return {Promise<Filter[]>} Filter preferences
     */
    this.getWidgetFilters = function(widgetID){
        return ZhPreferences.getWidgetPreferences('filter', widgetID).then(function parseFilterRow(preferences){
            preferences.forEach(function(preference){
                preference.FILTER_ROW = JSON.parse(preference.FILTER_ROW);
            });
            return preferences;
        });
    };

    /**
     * Retrieve local cached filters
     *
     * @since 3.0.0
     *
     * @param  {Query} query       Conditions to filter the preferences
     *
     * @return {Promise<Filter[]>} Filter preferences
     */
    this.getFilters = function(query){
        return ZhPreferences.getPreferences('filter', query);
    };

    /**
     * Persist filters
     *
     * @since 3.0.0
     *
     * @param  {Filter[]} filters  Filters to persist
     *
     * @return {Promise<Filter[]>} List of persisted filters
     */
    this.saveFilters = function(filters){
        var cloneFilters = filters.map(function(filter){ return ZhUtil.clearRow(filter); });
        return ZhPreferences.savePreferences('filter', cloneFilters).then(function(response){
            cloneFilters.forEach(function(clone, index){
                filters[index].ID = clone.ID;
                filters[index].__is_new = clone.__is_new;
                filters[index].changes = clone.changed;
            });

            return ZHPromise.when(response);
        });
    };

    /**
     * Delete filters
     *
     * @since 3.0.0
     *
     * @param  {Filter[]} filters  Filters to delete
     *
     * @return {Promise<Filter[]>} List of deleted filters
     */
    this.deleteFilters = function(filters){
        var cloneFilters = filters.map(function(filter){ return ZhUtil.clearRow(filter); });
        return ZhPreferences.deletePreferences('filter', cloneFilters);
    };

    function applyOriginalFilterRow(widget, filter){
        filter.FILTER_ROW = Util.clone(originalFilterRows[widget.id] || {});
    }

    /**
     * Apply original filter
     *
     * @since 3.0.0
     *
     * @param  {Widget} widget Widget to apply the original filter on
     *
     * @return {Object[]}      Rows returned using the filter
     */
    this.applyOriginalFilter = function(widget){
        applyOriginalFilterRow(widget, originalFilter);
        return this.applyFilter(widget, originalFilter);
    };

    /**
     * Set the filter row used when applying the Original filter
     *
     * @since 3.4.0
     *
     * @param {String} widgetId Widget identifier to set the original filter row
     * @param {Object} filter   Filter row to create the original filter
     */
    this.setOriginalFilter = function(widgetId, filter){
        originalFilterRows[widgetId] = Util.clone(filter);
    };

    /**
     * Get the filter row used when applying the Original filter
     *
     * @since 3.4.0
     *
     * @return {Object} filter  Filter row used to create the original filter
     */
    this.getOriginalFilter = function(widgetId, widget){
        var clearedFilter = ZhUtil.clearRow(originalFilterRows[widgetId] || {});
        var originalFilter = {
            ID: 'ORIGINAL',
            LABEL_TO_PREFERENCE: 'Original',
            FILTER_ROW: Util.clone(clearedFilter)
        };
        return originalFilter;
    };

    function getFilterConditions(filters){
        return filters.filter(function(filter){
            return filter.name.indexOf('SEARCHLIST_') === -1;
        }).map(function(filter){
            filter.isCustomFilter = true;
            return filter;
        });
    }

    function getSearchConditions(filters){
        var searchFields = Util.clone(filters);
        searchFields = searchFields.filter(function(filter){
            return ~filter.name.indexOf('SEARCHLIST_');
        });

        return searchFields.map(function(filter){
            filter.name = filter.name.replace('SEARCHLIST_', '');
            return filter;
        });
    }

    function triggerBeforeFilterApplyEvent(widget, preference){
        if(widget.beforeFilterApply){
            return widget.beforeFilterApply({
                'preference': preference
            });
        }
    }

    function triggerAfterFilterApplyEvent(widget, preference, filters){
        if(widget.afterFilterApply){
            widget.afterFilterApply({
                'preference': preference,
                'filters': filters
            });
        }
    }

    function prepareFilterRow(preference){
        var filterRow = preference.FILTER_ROW || {};
        filterRow = typeof preference.FILTER_ROW === 'string'?
            JSON.parse(preference.FILTER_ROW) : preference.FILTER_ROW;
        preference.FILTER_ROW = filterRow;
    }

    function applyFilterInFilterPopup(widget, filterRow){
        self.createFilterWidget(widget, filterRow);
    }

    function mergeFilters(baseFilters, newFilters){
        var result = baseFilters.concat(newFilters).reduce(function(prev, current, index){
            var fieldName = current.name;
            if(fieldName in prev.keys){
                var replaceIndex = prev.keys[fieldName];
                prev.filters[replaceIndex] = current;
            } else {
                prev.keys[fieldName] = index;
                prev.filters.push(current);
            }
            return prev;
        }, {
            filters: [],
            keys: {}
        });
        return result.filters;
    }

    function buildDataSourceFilter(widget, filters){
        var filterConditions = getFilterConditions(filters);
        var originalFilters = getOriginalDataSourceFilters(widget.dataSourceFilter);
        return mergeFilters(originalFilters, filterConditions);
    }

    function applyFilterInDataSource(widget, filters){
        widget.dataSourceFilter = buildDataSourceFilter(widget, filters);

        var searchConditions = getSearchConditions(filters);
        if(widget.setSearchCriteria){
            widget.setSearchCriteria(searchConditions);
        }
        return searchConditions;
    }

    function setAppliedFilter(widget, preference){
        widget.appliedFilter = preference;
    }

    function applyFilterInWidget(widget, fields, filterRow, preference){
        self.updateFilterInfo(widget, fields, filterRow, preference, {});
        setAppliedFilter(widget, preference);
    }

    /**
     * Apply filter
     *
     * @since 3.0.0
     *
     * @param  {Widget}  widget              Widget to apply the original filter on
     * @param  {Filter}  preference          Preference to be applied on widget
     * @param  {Boolean} [reloadWidget=true] Flag to cancel reload in widget
     *
     * @return {Object[]}          Rows returned using the filter
     */
    this.applyFilter = function(widget, preference, reloadWidget){
        reloadWidget = reloadWidget !== false;
        prepareFilterRow(preference);
        var event;

        if(reloadWidget){
            event = triggerBeforeFilterApplyEvent(widget, preference);
        }

        var filterRow = preference.CUSTOM_FILTER_ROW || preference.FILTER_ROW;

        var fields = self.factoryFields(null, widget.fields).filter(function removeInvisibleFields(field){
            return field.isVisible;
        });
        var filters = ZhFilterBuilder.buildFilter(fields, filterRow);

        if(reloadWidget){
            if(!event || !event.isDefaultPrevented()){
                var searchConditions = applyFilterInDataSource(widget, filters);
                var searchCriteria = buildSearchCriteria(searchConditions);

                return widget.reload(ZhUtil.getLikeAllFilter(widget, searchCriteria)).then(function(){
                    if (widget.dataSource) {
                        widget.dataSource.clearCheckedRows();
                    }
                    applyFilterInWidget(widget, fields, filterRow, preference);
                    triggerAfterFilterApplyEvent(widget, preference, filters);
                    applyFilterInFilterPopup(widget, preference.FILTER_ROW);

                    return ZHPromise.when();
                }.bind(this));
            } else {
                return ZHPromise.reject();
            }
        } else {
            applyFilterInDataSource(widget, filters);
            applyFilterInWidget(widget, fields, filterRow, preference);
            applyFilterInFilterPopup(widget, preference.FILTER_ROW);
            return ZHPromise.when();
        }
    };

    /**
     * Update the applied filter information
     *
     * @since 3.0.0
     *
     * @param  {Widget}  widget         Widget to update the filter information
     * @param  {Field[]} fields         List of fields used to create the filter
     * @param  {Object}  filterRow      Row representing the filter configuration
     * @param  {Filter}  preference     Filter currently applied
     * @param  {Object}  searchCriteria Search criteria currently applied
     */
    this.updateFilterInfo = function(widget, fields, filterRow, preference, searchCriteria){
        var filterInfo;
        var filterName = preference.LABEL_TO_PREFERENCE;

        if(((preference.ID === 'ORIGINAL' || preference.ID === 'EDITED') && !filterName) || preference.forceBuild){
            filterInfo = ZhFilterDescriptionBuilder.buildFilterInfo(fields, filterRow, searchCriteria, widget);
        } else {
            filterInfo = ZhFilterDescriptionBuilder.buildFilterAppliedInfo(filterName);
        }

        widget._filterInfo = filterInfo;
    };

    function trimPercentageSymbol(value){
        if(value.charAt(0) === '%'){
            value = value.substr(1);
        }

        if(value.charAt(value.length - 1) === '%'){
            value = value.substr(0, value.length - 1);
        }

        return value;
    }

    function buildSearchCriteria(conditions){
        var searchCriteria = {};
        conditions.forEach(function(condition){
            searchCriteria[condition.name] = trimPercentageSymbol(condition.value);
        });

        return searchCriteria;
    }

    function getOriginalDataSourceFilters(dataSourceFilter){
        return dataSourceFilter.filter(function(filter){
            return !filter.isCustomFilter;
        });
    }

    function removeConflictingProperties(field){
        var conflictingProperties = ['selectWidget', 'bindedEvents'];
        conflictingProperties.forEach(function(property){
            delete field[property];
        });
    }

    function factoryFields(gridFields, editing){
        var cloneField = function(field){
            var fieldClone = self.cloneObj(field);
            if(editing){
                fieldClone.fieldGroup = 'FILTER_FIELDS';
            }
            fieldClone.dataSource = field.dataSource;
            fieldClone.dataSourceFilter = field.dataSourceFilter;
            return fieldClone;
        };
        return gridFields.filter(function(field){
            return !Util.isUndefined(field.template);
        }).map(cloneField);
    }

    function factorySearchFields(gridFields){
        var factoredFields = [
            {
                'id': 'SEARCHLIST__ALL_MAP',
                'label': 'All Fields',
                'name': 'SEARCHLIST__ALL_MAP',
                'class': 12,
                'template': 'field/text-edit.html',
                'isVisible': true,
                'showOnList': true,
                'descending': true,
                'fieldGroup': 'MORE_FIELDS'
            }
        ];

        var buildSearchListField = function(field){
            var fieldClone = self.cloneObj(field);
            fieldClone.id = 'SEARCHLIST_' + fieldClone.id;
            fieldClone.name = 'SEARCHLIST_' + fieldClone.name;
            fieldClone.class = 6;
            fieldClone.isVisible = true;
            fieldClone.readOnly = false;
            fieldClone.validations = {};
            fieldClone.template = 'field/text-edit.html';
            fieldClone.fieldGroup = 'MORE_FIELDS';
            return fieldClone;
        };

        return factoredFields.concat(gridFields.filter(function(field){
            return !field.skipSearch && Util.isUndefined(field.template);
        }).map(buildSearchListField));
    }

    function mergeFieldNewProperties(field, filterProperties){
        var keys = Object.keys(filterProperties);
        keys.forEach(function(property){
            field[property] = filterProperties[property];
        });
        if(field.dataSource && !(field.dataSource instanceof DataSource)){
            field.dataSource = metaDataFactory.factoryDataSource(field.dataSource, field.dataSourceFilter, field);
        }
        delete field.filterProperties;
        return field;
    }

    function removeEventListener(field, eventName, eventIndex){
        eventIndex = eventEngine.bindEvent(field, eventName);
        eventEngine.unbindEvent(field, eventName, eventIndex);
        eventIndex--;
        eventEngine.unbindEvent(field, eventName, eventIndex);
    }

    /**
     * Factory fields used on filter widget
     *
     * @since 3.0.0
     *
     * @param  {Widget}   parentWidget Widget to be the parent of the fields
     * @param  {Fields[]} gridFields   Fields from the widget to use the filter
     * @param  {Boolean}  editing      Filter configuration being edited
     *
     * @return {Fields[]}              Factored fields
     */
    this.factoryFields = function(parentWidget, gridFields, editing){
        var fields = factoryFields(gridFields, editing);
        var searchFields = factorySearchFields(gridFields);
        var buildFilterProperties = function(field){
            if(field.filterProperties){
                field = mergeFieldNewProperties(field, field.filterProperties);
            }
            removeConflictingProperties(field);
            removeEventListener(field, "FieldOnChange");
            return metaDataFactory.fieldFactory(field, parentWidget);
        };

        return fields.concat(searchFields).map(buildFilterProperties);
    };

    /**
     * Export filter file
     *
     * @since 3.0.0
     *
     * @param  {Filter} filter  Filter to export
     *
     * @return {Promise<Filter>} Filter exported
     */
    this.downloadPreferenceFile = function(filter){
        var fileName = filter.LABEL_TO_PREFERENCE.latinise().replace(/[^a-z,0-9 ]/ig,'').replace(/(?:^\w|[A-Z]|\b\w|\s+)/g, function(match, index){
            if(+match === 0) return "";
            return index == 0 ? match.toLowerCase() : match.toUpperCase();
        }) + '.filter';

        var filterToExport = {
            'ID': filter.ID,
            'LABEL_TO_PREFERENCE': filter.LABEL_TO_PREFERENCE,
            'WIDGET_ID': filter.WIDGET_ID,
            'FILTER_ROW': ZhUtil.clearRow(filter.FILTER_ROW),
            'LAST_MODIFIED_DATE': filter.LAST_MODIFIED_DATE
        };

        return ZhPreferences.downloadPreferenceFile(filterToExport, fileName);
    };

    /**
     * Validade filter to import
     *
     * @since 3.0.0
     *
     * @param  {Filter} filter              Filter to import
     * @param  {Widget} configWidget        Widget to validate
     * @param  {Object[]} filterDataSource  Existing filters data (optional)
     *
     * @return {Boolean} Filter valid
     */
    this.validateFilterImport = function(filter, configWidget, filterDataSource){
        filterDataSource = filterDataSource || configWidget.dataSource.data;

        if(configWidget.listWidget.id != filter.WIDGET_ID){
            MessageService.showMessageByCode('zh-filter-preferences#import_error_widget_id');
            return false;
        }

        var filtersWithSameId = filterDataSource.filter(function(row){
            return row.ID == filter.ID;
        });

        if(filtersWithSameId.length > 0){
            MessageService.showMessageByCode('zh-filter-preferences#import_error_id_violation');
            return false;
        }

        var filtersWithSameLabel = filterDataSource.filter(function(row){
            return row.LABEL_TO_PREFERENCE == filter.LABEL_TO_PREFERENCE;
        });

        if(filtersWithSameLabel.length > 0){
            MessageService.showMessageByCode('zh-filter-preferences#import_error_name_violation');
            return false;
        }

        return true;
    };

    function openFilterWidget(widget, rowToBeSet) {
        return createFilterWidget(widget, rowToBeSet).then(function(widgetToBeShown) {
            widget.activate();
            ScreenService.openPopup(widgetToBeShown);
        });
    }
    this.openFilterWidget = openFilterWidget;

    var filterWidgets = {};

    function forceSelectDataSourceToLoad(field, row){
        var data = Util.getNestedProperty(field, ['dataSource', 'data', 'length']);
        var currentRow = row[field.name];

        if(field.dataSource && !data && currentRow){
            var fieldName = field.valueField || field.name;
            field.dataSource.load([{name: fieldName, value: currentRow, operator: "IN"}]);
        }
    }

    function forceFieldDataSourceToLoad(widget){
        widget.fields.forEach(function(field){
            forceSelectDataSourceToLoad(field, widget.currentRow);
        });
    }

    function createFilterWidget(widget, rowToBeSet) {
        return ApplicationContext.ZhFilterCreateFilterConditionsController.getCreateFilterConditionsWidgetForFilterPopup(widget).then(function(filterWidget) {
            if(!widget.filterInitialized){
                filterWidget.listWidget = widget;
                widget.filterInitialized = true;
            }

            filterWidget.label = ScreenService.i18n('Filter');

            var currentRow = rowToBeSet || widget.filterRowData;
            if(!currentRow) {
                currentRow = widget.appliedFilter ? widget.appliedFilter.FILTER_ROW : {};
            }

            filterWidget.currentRow = filterWidget.currentRow || {};
            Object.keys(currentRow).forEach(function(fieldName){
                filterWidget.currentRow[fieldName] = ZhUtil.evaluate(currentRow[fieldName]);
            });

            forceFieldDataSourceToLoad(filterWidget);

            filterWidget.actions[0].isVisible = true;
            filterWidget.actions[1].isVisible = true;
            filterWidget.actions[2].isVisible = false;
            filterWidget.actions[3].isVisible = false;

            filterWidgets[widget.id] = filterWidget;
            return ZHPromise.when(filterWidget);
        }.bind(this));
    }
    this.createFilterWidget = createFilterWidget;

    function isCopiable(object){
        var isCopiable = Util.isUndefinedOrNull(object);
        if(!isCopiable){
            var constr = object.constructor;
            var types = [String, Number, Array, Object, Boolean];
            isCopiable = types.some(function(type){
                return type().constructor === constr;
            });
        }
        return isCopiable;
    }

    /**
     * Clone the object avoiding first level non JavaScript basic and undefined types.
     *
     * @since ^3.14
     *
     * @param {Object} object The object to be copied.
     *
     * @return {Object} Returns the new object.
     */
    this.cloneObj = function(object){
        var clone = {};
        Object.keys(object).forEach(function(property){
            if(isCopiable(object[property])){
                clone[property] = ZhUtil.clearRow(object[property]);
            }
        });
        return clone;
    };

    function hasPopupOpened(){
        var isMobiScrollOpen = $('.mbsc-mobiscroll').length > 0;
        return !ScreenService.isSideMenuOpen() && !isMobiScrollOpen && !ScreenService.isAlertShown();
    }

    function isFilterPreferencesActive(widget){
        return Util.get(widget, ['floatingControl', 'customizationAction', 'filter']) !== false;
    }

    function isSearchActionActive(widget){
        return Util.get(widget, ['floatingControl', 'searchAction']) !== false;
    }

    function openWidgetFilter(){
        var activeWidget = templateManager.container.getActiveWidget();
        if(hasPopupOpened() && isFilterPreferencesActive(activeWidget) && ZhPreferences.validateWidget(activeWidget)){
            openFilterWidget(activeWidget);
        }

        return false;
    }

    function openWidgetSearch(){
        var activeWidget = templateManager.container.getActiveWidget();
        var searchIsEnabled = isFilterPreferencesActive(activeWidget) || isSearchActionActive(activeWidget);
        if(searchIsEnabled && hasPopupOpened() && ZhPreferences.validateWidget(activeWidget)){
            activeWidget.openSearchAction();
        }

        return false;
    }

    templateManager.projectPromise.then(function(){
        templateManager.project.shortcutKeys = templateManager.project.shortcutKeys || {};
        var openFilterKey = templateManager.project.shortcutKeys.openWidgetFilter || 'mod+l';
        var openSearchKey = templateManager.project.shortcutKeys.openWidgetSearch || 'mod+f';

        ZhKeyboardService.unbind(openFilterKey);
        ZhKeyboardService.unbind(openSearchKey);

        ZhKeyboardService.bind(openFilterKey, openWidgetFilter);
        ZhKeyboardService.bind(openSearchKey, openWidgetSearch);
    });
}

Configuration(function(ContextRegister){
    ContextRegister.register('ZhFilterPreferences', ZhFilterPreferences);
});

// FILE: src/controllers/ZhFilterWidgetsBuilder.js
/**
 * Service to create widgets to handle filters
 *
 * @class
 * @since 3.0.0
 *
 * @param {RestEngine}      restEngine      Service to make http requests
 * @param {TemplateManager} templateManager Service to handle templates
 * @param {MetaDataFactory} metaDataFactory Service to factory models from metadata
 */
function ZhFilterWidgetsBuilder(restEngine, templateManager, metaDataFactory, ZhFilterPreferences){
    var widgetsJson = {};

    function getWidgetJSON(widgetName){
        var widgetPath = 'zh-filter-preferences#json/widgets/'+widgetName+'.json';
        if(!widgetsJson[widgetPath]){
            widgetsJson[widgetPath] = restEngine.requestMetaData(widgetPath).then(function(response){
                return response.data;
            });
        }
        return widgetsJson[widgetPath];
    }

    /**
     * Retrieve a factored widget
     *
     * @since 3.0.0
     *
     * @param  {String} widgetName Widget name
     *
     * @return {Widget}            Factored widget
     */
    this.factoryWidget = function(widgetName){
        return getWidgetJSON(widgetName).then(function(widget){
            widget = Util.clone(widget);
            return metaDataFactory.widgetFactory(widget, templateManager.container);
        });
    };

    /**
     * Retrieve factored filter configuration widget
     *
     * @since 3.0.0
     *
     * @return {Widget} Filter configuration widget
     */
    this.factoryFilterConfigurationWidget = function(){
        return this.factoryWidget('config');
    };

    /**
     * Retrieve factored create filter widget
     *
     * @since 3.0.0
     *
     * @return {Widget} Create filter widget
     */
    this.factoryCreateFilterWidget = function(){
        return this.factoryWidget('createFilter');
    };

    /**
     * Retrieve factored import filter widget
     *
     * @since 3.0.0
     *
     * @return {Widget} Import filter widget
     */
    this.factoryImportFilterWidget = function(){
        return this.factoryWidget('importFilter');
    };

    function copyFieldGroups(fieldGroups){
        var copiedObj = (fieldGroups || []).map(function(fieldGroup){
            return Util.clone(fieldGroup, ['__proto__', 'rows', 'widget']);
        });
        return copiedObj;
    }

    function buildFilterWidgetFieldGroups(filterWidget, widget){
        var fieldGroups = (filterWidget.fieldGroups || []).concat(copyFieldGroups(widget.fieldGroups));
        var moreFieldsFieldGroup = [];
        fieldGroups = fieldGroups.filter(function(fieldGroup){
            var isMoreFieldFieldGroup = fieldGroup.name === 'MORE_FIELDS';
            if(isMoreFieldFieldGroup){
                moreFieldsFieldGroup = fieldGroup;
                moreFieldsFieldGroup.isVisible = !widget.hideMoreFieldsOnFilter;
            }
            return !isMoreFieldFieldGroup;
        });
        filterWidget.fieldGroups = fieldGroups.concat(moreFieldsFieldGroup);
    }

    /**
     * Retrieve factored build filters conditions widget
     *
     * @since 3.0.0
     *
     * @return {Widget} Build filters conditions widget
     */
    this.factoryBuildFiltersConditionsWidget = function(baseWidget){
        return getWidgetJSON('createFilterConditions').then(function(widgetJson){
            widgetJson = Util.clone(widgetJson);
            buildFilterWidgetFieldGroups(widgetJson, baseWidget);
            var widget = metaDataFactory.widgetFactory(widgetJson, templateManager.container);
            widget.fields = widget.fields.concat(ZhFilterPreferences.factoryFields(widget, baseWidget.fields));
            widget._filteredWidgetId = baseWidget.id;
            return widget;
        });
    };

}

Configuration(function(ContextRegister){
    ContextRegister.register('ZhFilterWidgetsBuilder', ZhFilterWidgetsBuilder);
});

// FILE: src/controllers/ZhFilterCreateFilterConditionsController.js
/**
 * Create filter conditions widget controller
 *
 * @class
 * @since 3.0.0
 *
 * @param {ZhFilterPreferences}    ZhFilterPreferences    Service to handle filter preferences methods
 * @param {ZhFilterWidgetsBuilder} ZhFilterWidgetsBuilder Service to create widgets to handle filters
 * @param {MessageService}         MessageService         Service to show messages
 * @param {ScreenService}          ScreenService          Service to manipulate screen
 * @param {ZHPromise}              ZHPromise              Service to handle promises
 * @param {ZhUtil}                 ZhUtil                 Service that contains utilities methods
 */
function ZhFilterCreateFilterConditionsController(ZhFilterPreferences, ZhFilterWidgetsBuilder, MessageService, ScreenService, ZHPromise, ZhUtil, templateManager){

    var createFilterConditionsWidgetPromise = {};

    /**
     * Retrieve widget to create filter conditions
     *
     * @since 3.0.0
     *
     * @return {Widget} Widget to create filter conditions
     */
    this.getCreateFilterConditionsWidget = function(widget){
        var widgetId = widget.id;
        if(!createFilterConditionsWidgetPromise[widgetId]){
            createFilterConditionsWidgetPromise[widgetId] = ZhFilterWidgetsBuilder.factoryBuildFiltersConditionsWidget(widget);
        }

        return createFilterConditionsWidgetPromise[widgetId];
    };

    function saveFilterFields(widget){
        var allowedFields = widget.fields.map(function(field){
            return ZhFilterPreferences.cloneObj(field);
        });

        widget.__originalFilterStatus = allowedFields;
        widget.__originalCurrentRow = ZhFilterPreferences.cloneObj(ZhUtil.clearRow(widget.currentRow));
    }

    /**
     * Retrieve widget to create filter conditions with filter popup configuration
     *
     * @since 3.0.0
     *
     * @return {Widget} Widget to create filter conditions
     */
    this.getCreateFilterConditionsWidgetForFilterPopup = function(widget){
        return this.getCreateFilterConditionsWidget(widget).then(function(createFilterConditionsWidget){
            createFilterConditionsWidget.actions[0].isVisible = true;
            createFilterConditionsWidget.actions[1].isVisible = true;
            createFilterConditionsWidget.actions[2].isVisible = false;
            createFilterConditionsWidget.actions[3].isVisible = false;
            var labelField = createFilterConditionsWidget.getField('LABEL_TO_PREFERENCE');

            if(labelField){
                labelField.isVisible = false;
            }

            saveFilterFields(createFilterConditionsWidget);

            return createFilterConditionsWidget;
        });
    };


    /**
     * Retrieve widget to create filter conditions with edit preference configuration
     *
     * @since 3.0.0
     *
     * @return {Widget} Widget to create filter conditions
     */
    this.getCreateFilterConditionsWidgetForEditPreference = function(widget){
        return this.getCreateFilterConditionsWidget(widget).then(function(createFilterConditionsWidget){
            createFilterConditionsWidget.actions[0].isVisible = false;
            createFilterConditionsWidget.actions[1].isVisible = false;
            createFilterConditionsWidget.actions[2].isVisible = true;
            createFilterConditionsWidget.actions[3].isVisible = true;

            var labelField = createFilterConditionsWidget.getField('LABEL_TO_PREFERENCE');

            if(labelField){
                labelField.isVisible = true;
            }

            saveFilterFields(createFilterConditionsWidget);

            return createFilterConditionsWidget;
        });
    };

    /**
     * Save filter and apply
     *
     * @since 3.0.0
     *
     * @param  {Widget} createFilterConditionsWidget Widget that has the filter conditions to be saved
     *
     * @return {Promise}                             Promise resolved when saved filter is applied
     */
    this.saveAndApplyFilter = function(createFilterConditionsWidget){

        var clearRequiredFieldErrors = function(invalidFields){
            invalidFields.shift();
            invalidFields.forEach(function(invalidField){
                var fieldName;
                invalidField = invalidField.filter(function(invalidItem){
                    fieldName = invalidItem.fieldName;
                    return invalidItem.errorMessage != "Required field";
                });
                if(!invalidField.length){
                    delete createFilterConditionsWidget.currentRow.$error[fieldName];
                }
            });
        };

        if(createFilterConditionsWidget.isValid(clearRequiredFieldErrors)){
            var currentRow = createFilterConditionsWidget.currentRow;
            var filter = createFilterConditionsWidget.preference;

            filter.LABEL_TO_PREFERENCE = currentRow.LABEL_TO_PREFERENCE;
            filter.FILTER_ROW = ZhUtil.clearRow(currentRow);
            delete filter.FILTER_ROW.LABEL_TO_PREFERENCE;
            filter.FILTER_ROW = JSON.stringify(filter.FILTER_ROW);

            var configWidget = createFilterConditionsWidget.configWidget;
            var listWidget = configWidget.listWidget;

            return ZhFilterPreferences.saveFilters([filter]).then(function(){
                filter.FILTER_ROW = JSON.parse(filter.FILTER_ROW);
                MessageService.successNotificationByCode('zh-filter-preferences#save_success');
                ScreenService.closePopup(true);

                if(createFilterConditionsWidget.isValid()){
                    filter.forceBuild = true;
                    listWidget.filterRowData = currentRow;
                    ZhFilterPreferences.applyFilter(listWidget, filter);
                    listWidget.activate();
                } else {
                    listWidget.openFilterWidget(createFilterConditionsWidget.currentRow);
                }

                ZhFilterPreferences.updateFilters(listWidget.id, configWidget.dataSource);
            }).catch(function(error){
                MessageService.showMessageByCode('zh-filter-preferences#save_error', {
                    'ERROR': error.message
                });
            });
        } else {
            return ZHPromise.reject("Invalid params");
        }
    };

    function getRowToApply(appliedFilter, currentRow){
        var appliedRow = (appliedFilter && appliedFilter.FILTER_ROW) || {};
        var row = ZhUtil.clearRow(currentRow);
        Object.keys(appliedRow).forEach(function(key){
            // Do not replace evaluated values
            if(Util.isString(appliedRow[key]) && appliedRow[key].indexOf('@') === 0){
                row[key] = appliedRow[key];
            }
        });
        return row;
    }

    this.clearFilter = function(widget){
        widget.fields
            .filter(function(field){
                return !field.readOnly;
            })
            .forEach(function(field){
                if(field.template && field.template.indexOf('field/checkbox.html') !== -1){
                    widget.currentRow[field.name] = field.valueToFalse;
                    field.checked = false;
                    templateManager.updateTemplate();
                } else {
                    field.clearValue(field);
                }
                if(field.change){
                    field.change();
                }
            });
    };

    /**
     * Apply filter
     *
     * @since 3.0.0
     *
     * @param  {Widget} createFilterConditionsWidget Widget that has the filter conditions to be saved
     * @param  {String} [filterName] - The name of the filter being applied
     *
     * @return {Promise}                             Promise resolved when saved filter is applied
     */
    this.applyFilter = function(createFilterConditionsWidget, filterName){
        if(createFilterConditionsWidget.isValid()){
            var currentRow = createFilterConditionsWidget.currentRow;
            var listWidget = createFilterConditionsWidget.listWidget;
            delete currentRow.LABEL_TO_PREFERENCE;

            var filter = {
                'ID': 'EDITED',
                'FILTER_ROW': getRowToApply(listWidget.appliedFilter, currentRow)
            };

            if(filterName){
                filter.LABEL_TO_PREFERENCE = filterName;
            }

            return ZhFilterPreferences.applyFilter(listWidget, filter).then(function(){
                if(ScreenService.getPopupOpen() == createFilterConditionsWidget){
                    ScreenService.closePopup(true);
                    listWidget.activate();
                }
            });
        } else {
            if(ScreenService.getPopupOpen() != createFilterConditionsWidget){
                ScreenService.openPopup(createFilterConditionsWidget);
            }
            return ZHPromise.reject();
        }
    };

    function restoreFilterFields(widget){
        if(widget.__originalFilterStatus){
            widget.fields.forEach(function(field, index){
                var __field = widget.__originalFilterStatus.filter(function(fieldOriginal){
                    return fieldOriginal.name === field.name;
                })[0];
                Object.keys(__field).forEach(function(key){
                    widget.fields[index][key] = __field[key];
                });
            });
            widget.currentRow = widget.__originalCurrentRow;

            delete widget.__originalCurrentRow;
            delete widget.__originalFilterStatus;
        }
    }

    /**
     * Close widget popup and restore original status of filter widget
     *
     * @since 3.14.x
     *
     * @param  {Widget} createFilterConditionsWidget Widget that has the filter conditions to be restored
     *
     * @return {undefined}
     */
    this.closePopup = function(widget){
        restoreFilterFields(widget);
        ScreenService.closePopup();
        widget.listWidget.activate();
    };

}

Configuration(function(ContextRegister){
    ContextRegister.register('ZhFilterCreateFilterConditionsController', ZhFilterCreateFilterConditionsController);
});

// FILE: src/controllers/ZhFilterImportFilterController.js
/**
 * Import filter widget controller
 *
 * @class
 * @since 3.0.0
 *
 * @param {MessageService}                        MessageService                           Service to show messages
 * @param {ZhFilterPreferences}                   ZhFilterPreferences                      Service to handle filter preferences methods
 * @param {ZhPreferences}                         ZhPreferences                            Service to handle preferences persistance
 * @param {ZhFilterWidgetsBuilder}                ZhFilterWidgetsBuilder Service to create widgets to handle filters
 * @param {ZHPromise}                             ZHPromise                                Service to handle promises
 * @param {ScreenService}                         ScreenService                            Service to manipulate screen
 */
function ZhFilterImportFilterController(MessageService, ZhFilterPreferences, ZhPreferences, ZhFilterWidgetsBuilder, ZHPromise, ScreenService){

    var importFilterWidgetPromise = null;

    /**
     * Retrieve widget to import filter
     *
     * @since 3.0.0
     *
     * @return {Promise<Widget>}
     */
    this.getImportFilterWidget = function(){
        if(importFilterWidgetPromise === null){
            importFilterWidgetPromise = ZhFilterWidgetsBuilder.factoryImportFilterWidget();
        }

        return importFilterWidgetPromise;
    };

    /**
     * Import filter
     *
     * @since 3.0.0
     *
     * @param  {Widget} importFilterWidget Widget with the filter configuration
     *
     * @return {Promise}                      Promise resolved after the filter is imported
     */
    this.importPreference = function(importFilterWidget){
        if(!importFilterWidget.isValid()){
            return ZHPromise.reject({err: 'WIDGET_INVALID'});
        }

        var filterFileData = importFilterWidget.fields[0].value()[0].b64File;
        filterFileData = filterFileData.split('base64,')[1];
        filterFileData = window.atob(filterFileData);
        filterFileData = decodeURIComponent(escape(filterFileData));

        var filter = JSON.parse(filterFileData);

        var configWidget = importFilterWidget.configWidget;
        var listWidget = configWidget.listWidget;

        if(!ZhFilterPreferences.validateFilterImport(filter, configWidget)){
            return ZHPromise.reject({err: 'FILTER_FILE_INVALID'});
        }

        filter.__is_new = true;
        filter.__generate_id = false;
        filter.FILTER_ROW = JSON.stringify(filter.FILTER_ROW);

        return ZhPreferences.getUserID().then(function(userID){
            filter.USER_ID = userID;

            return ZhFilterPreferences.saveFilters([filter]).then(function(){
                filter.FILTER_ROW = JSON.parse(filter.FILTER_ROW);
                ScreenService.closePopup();
                ZhFilterPreferences.updateFilters(listWidget.id, configWidget.dataSource);
                configWidget.refreshWidget();
            }).catch(function(error){
                MessageService.showMessageByCode('zh-filter-preferences#save_error', {
                    'ERROR': error.message
                });
            });
        });
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('ZhFilterImportFilterController', ZhFilterImportFilterController);
});

// FILE: src/controllers/ZhFilterConfigWidgetController.js
/**
 * Filter configuration widget controller
 *
 * @class
 * @since 3.0.0
 *
 * @param {ZhPreferences}                            ZhPreferences                            Service to handle preferences persistance
 * @param {ZhFilterPreferences}                      ZhFilterPreferences                      Service to handle filter preferences methods
 * @param {ZhFilterWidgetsBuilder}                   ZhFilterWidgetsBuilder                   Service to create widgets to handle filters
 * @param {ScreenService}                            ScreenService                            Service to manipulate screen
 * @param {MessageService}                           MessageService                           Service to show messages
 * @param {ZHPromise}                                ZHPromise                                Service to handle promises
 * @param {ZhFilterCreateFilterConditionsController} ZhFilterCreateFilterConditionsController Controller to manipulate create filter conditions widget
 * @param {ZhFilterImportFilterController}           ZhFilterImportFilterController           Controller to manipulate import filter widget
 */
function ZhFilterConfigWidgetController(ZhPreferences, ZhFilterPreferences, ZhFilterWidgetsBuilder, ScreenService, MessageService, ZHPromise, ZhFilterCreateFilterConditionsController, ZhFilterImportFilterController){

    var configWidgetPromise = null;

    /**
     * Retrieve widget that handles filters configuration
     *
     * @since 3.0.0
     *
     * @param  {Widget}     listWidget Widget that is being used to create filters for
     * @param  {DataSource} dataSource DataSource to use on config widget
     *
     * @return {Promise<Widget>}       Widget that handles filters configuration
     */
    this.getConfigWidget = function(listWidget, dataSource){
        if(configWidgetPromise === null){
            configWidgetPromise = ZhFilterWidgetsBuilder.factoryFilterConfigurationWidget();
        }

        var userID = ZhPreferences.getUserID();

        return ZHPromise.all([userID, configWidgetPromise]).then(function(args){
            var userID = args[0];
            var configWidget = args[1];

            configWidget.listWidget = listWidget;
            configWidget.dataSourceFilter = [
                {'name': 'WIDGET_ID', 'operator': '=', 'value': listWidget.id}
            ];
            dataSource.data.forEach(function(row){
                row.APPLIED = row.ID == (listWidget.appliedFilter ? listWidget.appliedFilter.ID : 'ORIGINAL');
            });
            configWidget.dataSource = dataSource;

            return configWidget;
        });
    };

    var hideMoreFields = function(groups, hide){
        var moreFieldsFieldGroup= [];
        return groups.filter(function(fieldGroup){
            delete fieldGroup.rows;
            var isMoreFieldFieldGroup = fieldGroup.name === 'MORE_FIELDS';
            if(isMoreFieldFieldGroup){
                moreFieldsFieldGroup = fieldGroup;
                moreFieldsFieldGroup.isVisible = !hide;
            }
            return !isMoreFieldFieldGroup;
        }).concat(moreFieldsFieldGroup);
    };

    /**
     * Open widget to create a new filter
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that handles filters configuration
     *
     * @return {Promise}             Promise that is resolved when the widget is opened
     */
    this.openCreateFilterWidget = function(configWidget){
        var label = configWidget.currentRow.LABEL_TO_PREFERENCE;
        return ZhFilterCreateFilterConditionsController.getCreateFilterConditionsWidgetForEditPreference(configWidget.listWidget).then(function(createFilterConditionsWidget){
            var listWidget = configWidget.listWidget;

            createFilterConditionsWidget.label = ScreenService.i18n('Filter');
            createFilterConditionsWidget.configWidget = configWidget;

            if(!listWidget.filterInitialized){
                var fieldGroups = (createFilterConditionsWidget.fieldGroups || []).concat((listWidget.fieldGroups || []));
                createFilterConditionsWidget.fieldGroups = fieldGroups;
                listWidget.filterInitialized = true;
            }
            createFilterConditionsWidget.fieldGroups = hideMoreFields(createFilterConditionsWidget.fieldGroups, listWidget.hideMoreFieldsOnFilter);

            return ZhPreferences.getUserID().then(function(userID){
                var preference = {
                    '__is_new': true,
                    'ID': null,
                    'USER_ID': userID,
                    'LABEL_TO_PREFERENCE': label,
                    'WIDGET_ID': listWidget.id,
                    'LAST_MODIFIED_DATE': null
                };

                createFilterConditionsWidget.preference = preference;

                var labelField = createFilterConditionsWidget.getField('LABEL_TO_PREFERENCE');
                var fields = ZhFilterPreferences.factoryFields(createFilterConditionsWidget, listWidget.fields, true);
                createFilterConditionsWidget.fields = [labelField].concat(fields);
                labelField.isVisible = true;
                createFilterConditionsWidget.newRow();
                return ScreenService.openPopup(createFilterConditionsWidget);
            });
        });
    };


    /**
     * Open popup to edit a saved filter
     *
     * @since 3.0.0
     *
     * @param  {Filter} filter   Filter to edit
     *
     * @return {Promise<Widget>} Widget used to edit the filter
     */
    this.editFilter = function(configWidget, filter){
        var listWidget = configWidget.listWidget;
        return ZhFilterCreateFilterConditionsController.getCreateFilterConditionsWidgetForEditPreference(configWidget.listWidget)
            .then(function(createFilterConditionsWidget){
                createFilterConditionsWidget.preference = filter;
                createFilterConditionsWidget.label = ScreenService.i18n('Filter') + ' - ' + filter.LABEL_TO_PREFERENCE;
                createFilterConditionsWidget.configWidget = configWidget;
                createFilterConditionsWidget.fieldGroups= hideMoreFields(createFilterConditionsWidget.fieldGroups || [], listWidget.hideMoreFieldsOnFilter);

                var labelField = createFilterConditionsWidget.getField('LABEL_TO_PREFERENCE');
                var fields = ZhFilterPreferences.factoryFields(createFilterConditionsWidget, listWidget.fields, true);
                createFilterConditionsWidget.fields = [labelField].concat(fields);
                labelField.isVisible = true;

                createFilterConditionsWidget.currentRow = filter.FILTER_ROW;
                createFilterConditionsWidget.currentRow.LABEL_TO_PREFERENCE = filter.LABEL_TO_PREFERENCE;
                return ScreenService.openPopup(createFilterConditionsWidget);
            });
    };

    /**
     * Delete filters
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that contains the filters to delete
     *
     * @return {Promise}             Promise resolved after filters are deleted
     */
    this.deleteFilters = function(configWidget){
        var filtersToDelete = configWidget.getCheckedRows();

        var defaultFilter = filtersToDelete.filter(function(filter){
            return filter.DEFAULT_FILTER;
        });
        if(defaultFilter.length){
            MessageService.showMessageByCode('zh-filter-preferences#delete_error_default');

            return;
        }

        return MessageService.confirmMessageByCode('zh-filter-preferences#delete', {
            'NR_OF_FILTERS': filtersToDelete.length
        }).then(function(){
            return ZhFilterPreferences.deleteFilters(filtersToDelete).then(function(){
                configWidget.dataSource.clearCheckedRows();
                return ZhFilterPreferences.updateFilters(configWidget.listWidget.id, configWidget.dataSource);
            }).catch(function(error){
                return MessageService.getMessageByCode('zh-filter-preferences#'+error.message).then(function(errorMessage) {
                    return MessageService.showMessageByCode('zh-filter-preferences#delete_error', {
                        'ERROR': ScreenService.i18n(errorMessage)
                    });
                });
            });
        });
    };

    /**
     * Set a filter as the default preference
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that handles filters configuration
     * @param  {Filter} filter       Filter that it's default value has changed
     *
     * @return {Promise<Object>}     Widget default preferences
     */
    this.updateDefaultFilter = function(configWidget, filter){
        return ZhPreferences.changeDefaultPreferences(configWidget, 'DEFAULT_FILTER', filter);
    };

    /**
     * Apply Filter
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that handles filters configuration
     * @param  {Filter} filter       Filter to be applied
     *
     * @return {Promise<Object>}     Widget default preferences
     */
    this.updateAppliedFilter = function(configWidget, filter){
        var listWidget = configWidget.listWidget;

        return ZhFilterPreferences.applyFilter(listWidget, filter).then(function(){
            configWidget.dataSource.data.forEach(function(row){
                row.APPLIED = row.ID == filter.ID;
            });
        });
    };

    /**
     * Download preference as file
     *
     * @since 3.0.0
     *
     * @param  {Filter} filter       Filter to export
     *
     * @return {Promise<Filter>}     Filter exportado
     */
    this.downloadPreferenceFile = function(filter){
        return ZhFilterPreferences.downloadPreferenceFile(filter);
    };

    /**
     * Open widget to import a new filter from file
     *
     * @since 3.0.0
     *
     * @param  {Widget} configWidget Widget that handles filter configuration
     *
     * @return {Promise}             Promise that is resolved when the widget is opened
     */
    this.importPreferenceFile = function(configWidget){
        return ZhFilterImportFilterController.getImportFilterWidget().then(function(importFilterWidget){
            importFilterWidget.newRow();
            importFilterWidget.configWidget = configWidget;
            return ScreenService.openPopup(importFilterWidget);
        });
    };

}

Configuration(function(ContextRegister){
    ContextRegister.register('ZhFilterConfigWidgetController', ZhFilterConfigWidgetController);
});

// FILE: src/controllers/ZhFilterFloatingCardController.js
/**
 * Floating window controller
 *
 * @class
 * @since 3.0.0
 *
 * @param {$scope}              $scope              Floating window scope
 * @param {$timeout}            $timeout            AngularJS's wrapper for window.setTimeout
 * @param {ApplicationContext}  ApplicationContext  Service to retrieve Zeedhi services
 * @param {templateManager}     templateManager     Service to handle templates
 * @param {metaDataFactory}     metaDataFactory     Service to factory models from metadata
 * @param {ScreenService}       ScreenService       Service to handle screen
 * @param {Query}               Query               Service to build queries
 * @param {eventEngine}         eventEngine         Service to handle events
 * @param {ZHPromise}           ZHPromise           Service to handle promises
 * @param {ZhUtil}              ZhUtil              Service to provice utilities methods
 */
function ZhFilterFloatingCardController($scope, ApplicationContext, templateManager, metaDataFactory, ScreenService, Query, eventEngine, ZHPromise, ZhUtil){

    var widget = $scope.widget;
    var ZhFilterPreferences = ApplicationContext.ZhFilterPreferences;
    var ZhPreferences = ApplicationContext.ZhPreferences;
    var ZhFilterConfigWidgetController = ApplicationContext.ZhFilterConfigWidgetController;
    var ZhFilterCreateFilterConditionsController = ApplicationContext.ZhFilterCreateFilterConditionsController;
    var filterWidget;

    var dataSource = metaDataFactory.factoryDataSource({
        'name': 'zh-filter-preferences#/filter',
        'localStorage': false,
        'lazyLoad': true,
        'rest': true,
        'data': []
    });

    dataSource.afterLoadDataSource(function(){
        return ZhFilterPreferences.addIsDefaultFilterToFilters(dataSource.data, widget.id).then(function(filters){
            dataSource.data = filters.map(function(filter){
                filter.FILTER_ROW = JSON.parse(filter.FILTER_ROW);
                return filter;
            });

        });
    });

    $scope.customizationCard = {
        'label': 'Filters',
        'type': 'filter',
        'openFilterConfiguration': openFilterConfiguration,
        'refreshFilters': refreshFilters,
        'openFilterWidget': openFilterWidget,
        'applyOriginalFilter': applyOriginalFilter,
        'applyFilter': applyPreference,
        'dataSource': dataSource
    };

    ZhFilterPreferences.registerFilterDataSource(widget.id, $scope.customizationCard.dataSource);

    widget.openFilterWidget = openFilterWidget;
    widget.refreshFilters = refreshFilters;

    function updateFilterRow(filterRow, searchCriteria){
        var changed = false;
        filterRow = filterRow || {};
        Object.keys(searchCriteria).forEach(function(key){
            var name = 'SEARCHLIST_' + key;
            var filterValue = filterRow[name];
            if(!Util.isEmptyOrBlank(filterValue) && searchCriteria[key] !== filterValue){
                changed = true;
            }
            filterRow[name] = searchCriteria[key];
        });

        filterRow.changed = changed;
        return filterRow;
    }

    $scope.$watchCollection('widget.searchCriteria', function(searchCriteria){
        if(ZhPreferences.validateWidget(widget)){
            var filterRow = widget.appliedFilter && widget.appliedFilter.FILTER_ROW;
            filterRow = updateFilterRow(filterRow, searchCriteria || {});

            if(filterRow.changed){
                widget.appliedFilter = {
                    ID: 'EDITED',
                    FILTER_ROW: filterRow
                };
            }
            delete filterRow.changed;

            if(widget.appliedFilter){
                ZhFilterPreferences.updateFilterInfo(widget, widget.fields, filterRow, widget.appliedFilter, searchCriteria);
            }
        }
    });

    function openFilterConfiguration(){
        var dataSource = $scope.customizationCard.dataSource;
        return ZhFilterConfigWidgetController.getConfigWidget(widget, dataSource).then(function(configWidget){
            return ScreenService.openPopup(configWidget);
        });
    }

    function refreshFilters(){
        return ZhPreferences.getUserID().then(function(){
            var query = Query.build()
                .where('WIDGET_ID').equals(widget.id);

            return dataSource.load(query.where());
        });
    }

    function openFilterWidget(rowToBeSet){
        return ZhFilterPreferences.openFilterWidget(widget, rowToBeSet);
    }

    function applyOriginalFilter(){
        return ZhFilterPreferences.applyOriginalFilter(widget);
    }

    function applyPreference(filter, reloadWidget){
        ZhFilterPreferences.applyFilter(widget, filter, reloadWidget);
    }

    function getDefaultFilter(){
        return dataSource.data.filter(function(filter){
            return filter.DEFAULT_FILTER;
        }).shift();
    }

    function applyDefaultFilter(){
        var isLazyLoad = Util.getNestedProperty(widget, ['dataSource', 'lazyLoad']);
        var filter = getDefaultFilter() || ZhFilterPreferences.getOriginalFilter(widget.id);

        applyPreference(filter, !isLazyLoad);
    }

    function getDataSourceFilterRow(){
        var filterRow = {};
        (widget.dataSourceFilter || [])
            .filter(function removeCustoms(filter){
                return !filter.isCustomFilter;
            })
            .forEach(function parseToFilterRow(filter){
                filterRow[filter.name] = filter.value;
            });
        return filterRow;
    }

    function initOriginalFilter(){
        var widgetId = widget.id;
        var filter = ZhFilterPreferences.getOriginalFilter(widgetId);
        if(Util.isEmptyObject(filter.FILTER_ROW)){
            var filterRow = getDataSourceFilterRow();
            widget.fields.forEach(function(field){
                var hasFilterDefaultValue = field.filterProperties && field.filterProperties.hasOwnProperty('defaultValue');
                var defaultValue = hasFilterDefaultValue ? field.filterProperties.defaultValue : field.defaultValue;
                if(!Util.isUndefinedOrNull(defaultValue)){
                    filterRow[field.name] = defaultValue;
                }
            });
            ZhFilterPreferences.setOriginalFilter(widgetId, filterRow);
        }

    }

    function isFilterPreferencesActive(activeWidget){
        return Util.get(activeWidget || widget, ['floatingControl', 'customizationAction', 'filter']) !== false;
    }

    $scope.$on('$destroy', function() {
        delete widget.openFilterWidget;
        delete widget.refreshFilters;
        delete dataSource.afterLoadDataSourcelistener;
    });

    (function initFilterPreferences(){
        var filterIsActive = isFilterPreferencesActive();
        if(filterIsActive && ZhPreferences.validateWidget(widget)){
            ZhFilterPreferences.updateFilters(widget.id, $scope.customizationCard.dataSource).then(function(){
                if(Util.isUndefinedOrNull(widget.appliedFilter) && widget.filterInitialized !== true){
                    initOriginalFilter();
                    applyDefaultFilter();
                }
            });
        }
    })();

}


// FILE: src/directives/ZhFilterFloatingControl.js
var ZeedhiDirectives = angular.module('ZeedhiDirectives');

ZeedhiDirectives.directive('zhFloatingControl', ['$timeout', 'templateManager', ZhFilterFloatingControl]);

function ZhFilterFloatingControl($timeout, templateManager){

    function checkFloatingCardOverflow(e){
        var actionElement = $(e.target).closest('.float-action');
        var floatingControl = actionElement.parents('.zh-floating-control');
        var widgetContainer = actionElement.closest('.container');
        var actionTop, actionBottom;
        var widgetTop, widgetBottom;

        if(!actionElement.hasClass('opened')){
            actionElement.removeClass('fix-position-top');
            actionElement.removeClass('fix-position-bottom');

            actionTop = actionElement.offset().top;
            widgetTop = widgetContainer.offset().top;
            widgetBottom = widgetTop + widgetContainer.height();
            if(floatingControl.hasClass('pos-top')){
                actionTop = actionTop - 200 + actionElement.height();
                if(actionTop < widgetTop){
                    actionElement.addClass('fix-position-top');
                }
            } else {
                actionBottom = actionTop + 200;
                if(actionBottom > widgetBottom){
                    actionElement.addClass('fix-position-bottom');
                }
            }
        }
    }

    return {
        priority: 10,
        link: function($scope, element, attrs, controller){
            var widget = $scope.widget;
            if(widget.floatingControl === false){
                return;
            }

            var isFilterApplied = function(){
                return widget && widget.appliedFilter && (Object.keys(widget.appliedFilter.FILTER_ROW).length > 0);
            };

            var defaultOptions = {
                searchAction: Util.isUndefined(element.parent().attr('data-hide-search')) && Util.isUndefined(element.parent().attr('hide-search')),
                openSearch: false
            };
            widget.floatingControl = Util.extend(defaultOptions, widget.floatingControl);

            var isFilterAppliedWithoutOriginal = function(){
                return widget._filterInfo && widget._filterInfo.length > 0 && widget.appliedFilter && (widget.appliedFilter.ID != 'ORIGINAL');
            };

            widget.floatingControl.addStatus({
                priority: 20,
                classString: 'searching',
                icon: 'more',
                conditionFunction: function(){
                    return isFilterAppliedWithoutOriginal();
                },
                infoTemplate: function(){
                    var hideFilterValue = templateManager.project.hideFilterValue || widget.hideFilterValue;
                    var filterInfoValue = hideFilterValue ? '</b> {{info.filterName}}': ':</b> {{info.value}}';
                    if(isFilterApplied()){
                        widget.floatingControl.statuses[1].icon = 'search';
                        if(widget.floatingControl.customizationAction && widget.floatingControl.customizationAction.filter){
                            icon = 'filter';
                            method = 'openFilter';
                        } else {
                            icon = 'close-x';
                            method = 'clearFilter';
                        }
                        return '<div data-zh-touchstart="'+method+'()"> <div class="searching-control-info"><span data-zh-icon="'+icon+' no-border icon-white"></span><div class="searching-control-info-details"><span ng-repeat="info in widget._filterInfo"><b>{{info.label|i18n}}'+filterInfoValue+'</span></div></div><div class="searching-control-info-text" data-zh-bind-translate="\'Filter applied\'"></div></div>';
                    }
                    widget.floatingControl.statuses[1].icon = 'more';
                },
                scope: $scope
            });

            if(templateManager.project.showEmptyFilter){
                widget.floatingControl.addStatus({
                    priority: 30,
                    classString: 'emptyFilter',
                    icon: 'more',
                    conditionFunction: function(){
                        return !isFilterAppliedWithoutOriginal();
                    },
                    infoTemplate: '<div data-zh-touchstart="openFilter()"> <div class="searching-control-info"><span data-zh-icon="filter no-border icon-white"></span></div><div class="searching-control-info-text"></div></div>',
                    scope: $scope
                });
            }

            $scope.openSearchAction = function(){
                $timeout.cancel($scope._closeTimeout);
                if(!element.hasClass('opened')){
                    $scope.toggleActionMenu();
                }
                $scope.closeOpenedActions();
                var searchAction = element.find('.search-action').closest('.float-action');
                if(searchAction && searchAction.length > 0){
                    searchAction.find("span.search-action-icon").click();
                    Util.selectElement(searchAction.find('input'));
                }
            };
            widget.openSearchAction = $scope.openSearchAction;

            $scope.$on('$destroy', function() {
               delete widget.openSearchAction;
               widget.floatingControl.statuses = [];
            });

            $scope.openFilter = function(){
                $scope.widget.openFilterWidget();
            };

            $scope.clearFilter = function(){
                $scope.widget.searchCriteria = {};
            };

            $scope.togglePreferencesCard = function(e){
                checkFloatingCardOverflow(e);
                $scope.toggleAction(e);
            };

            $scope.clearSearch = function(event, fieldName){
                if(!fieldName){
                    widget.searchCriteria = {};
                } else {
                    delete widget.searchCriteria[fieldName];
                }

                if(event){
                    Util.selectElement($(event.target).closest(".floating-card-input").find("input"));
                }
            };

            $scope.toggleSearchFieldsList = function(event){
                if(!widget.floatingControl.customizationAction || !widget.floatingControl.customizationAction.filter){
                    var el = $(event.target).closest(".floating-card").find('.floating-card-search-field-select');
                    el.toggleClass("opened");

                    if(el.hasClass("opened")){
                        $(event.target).closest(".floating-card").find('.floating-card-select-field-options').removeClass("opened");
                        $(event.target).closest(".floating-card").find('.zh-select-search-floating').removeClass("opened");
                    }
                }
            };

            $scope.selectSearchField = function(field){
                $scope.searchField = field;
                element.find('.floating-card-search-field-select').removeClass('opened');
                element.find('input.zh-input-search-floating').focus();
            };

            $scope.filterFields = function(field){
                return field.aclIsVisible && !field.skipSearch && ((field.isVisible && field.showOnList) || field.showOnFilter);
            };

            $scope.getFields = function(widget) {
                return (widget.fields || []).filter($scope.filterFields);
            };

            if(widget.floatingControl.defaultSearchField){
                $scope.searchField = widget.getField(widget.floatingControl.defaultSearchField);
            }

            if (widget.floatingControl.openSearch) {
                $timeout(widget.openSearchAction, 0, false);
            }
        }
    };

}

// FILE: src/Config.js
(function(){

window.templateUrls = window.templateUrls || [];
window.metadataUrls = window.metadataUrls || [];
window.serviceUrls = window.serviceUrls || [];

var PACKAGE_NAME = 'zh-filter-preferences';

var hasTemplateUrl = Boolean(window.templateUrls.find(function(value){
    return value.package === PACKAGE_NAME;
}));

var hasMetadataUrl = Boolean(window.metadataUrls.find(function(value){
    return value.package === PACKAGE_NAME;
}));

var hasServiceUrl = Boolean(window.serviceUrls.find(function(value){
    return value.package === PACKAGE_NAME;
}));

if(!hasTemplateUrl){
    window.templateUrls.push({
        package: PACKAGE_NAME,
        baseUrl: 'bower_components/zh-filter-preferences/dist/templates/'
    });
}

if(!hasMetadataUrl){
    window.metadataUrls.push({
        package: PACKAGE_NAME,
        baseUrl: 'bower_components/zh-filter-preferences/dist/assets/'
    });
}

if(!hasServiceUrl){
    window.serviceUrls.push({
        package: PACKAGE_NAME,
        baseUrl: '../backend/service/index.php/zh-preferences'
    });
}

})();
